import java.io.*;

/**
 * Reads one bit at a time from a file.
 *
 * @author Stefan Nilsson
 * @version 2017-01-24
 */
public class BitFileReader {
    private BufferedInputStream in;
    private int currentByte; // -1 if no more data
    private int bitPos;      // position in currentByte

    /**
     * Creates a BitFileReader by opening a connection to an actual file,
     * the file named by the File object file in the file system.
     */
    public BitFileReader(File file) throws IOException {
        in = new BufferedInputStream(new FileInputStream(file));
        currentByte = in.read();
        bitPos = 7;
    }

    /** Returns true if this reader has another bit in its input. */
    public boolean hasNextBit() {
        return currentByte != -1 && in != null;
    }

    /** Reads a single bit. */
    public int nextBit() throws IOException {
        int res = (currentByte>>bitPos) & 1;
        --bitPos;
        if (bitPos < 0) {
            currentByte = in.read(); // -1 if end of file has been reached
            bitPos = 7;
        }
        return res ;
    }

    /** Closes this reader. */
    public void close() throws IOException {
        if (in != null) {
            in.close();
        }
    }

    /**
     * Unit test.
     * Usage: java -ea BitFileReader
     */
    public static void main(String[] args) throws IOException {
        File temp;
        BitFileReader reader;
    
        // Empty file
        temp = File.createTempFile("testBitFileReaderEmpty", "txt");
        temp.deleteOnExit();

        reader = new BitFileReader(temp);
        assert !reader.hasNextBit();       
        reader.close();

        // File containing "AB"
        temp = File.createTempFile("testBitFileReaderAB", "txt");
        temp.deleteOnExit();
        BufferedWriter out = new BufferedWriter(new FileWriter(temp));  
        out.write("AB"); // AB = 0100 0001 0100 0010
        out.close();

        reader = new BitFileReader(temp);
        assert reader.hasNextBit();
        assert reader.nextBit() == 0;
        assert reader.hasNextBit();
        assert reader.nextBit() == 1;
        assert reader.nextBit() == 0;
        assert reader.nextBit() == 0;

        assert reader.nextBit() == 0;
        assert reader.nextBit() == 0;
        assert reader.nextBit() == 0;
        assert reader.hasNextBit();
        assert reader.nextBit() == 1;

        assert reader.hasNextBit();
        assert reader.nextBit() == 0;
        assert reader.nextBit() == 1;
        assert reader.nextBit() == 0;
        assert reader.nextBit() == 0;

        assert reader.nextBit() == 0;
        assert reader.nextBit() == 0;
        assert reader.nextBit() == 1;
        assert reader.hasNextBit();
        assert reader.nextBit() == 0;
        assert !reader.hasNextBit();
        reader.close();
    }
}
